@@ -3,11 +3,22 @@ from pathlib import Path
 
 import h5py
 import matplotlib.pyplot as plt
+import matplotlib.animation as animation
 import numpy as np
 
 import emout.plot as emplt
 import emout.utils as utils
 from emout.utils import InpFile, UnitConversionKey, Units, DataFileInfo, RegexDict
+from .util import t_unit
+
+
+def none_unit(out):
+    return utils.UnitTranslator(
+        1,
+        1,
+        name='',
+        unit=''
+    )
 
 
 class Emout:
@@ -23,13 +34,13 @@ class Emout:
 
     name2unit = RegexDict({
         r'phisp': lambda self: self.unit.phi,
-        r'nd[0-9]+p': lambda self: self.unit.n,
+        r'nd[0-9]+p': none_unit,
         r'rho': lambda self: self.unit.rho,
         r'rhobk': lambda self: self.unit.rho,
         r'j.*': lambda self: self.unit.J,
         r'b[xyz]': lambda self: self.unit.H,
         r'e[xyz]': lambda self: self.unit.E,
-        r't': lambda self: self.unit.t,
+        r't': t_unit,
         r'axis': lambda self: self.unit.length,
     })
 
@@ -314,7 +325,7 @@ class Data(np.ndarray):
         管理するデータのxyz方向それぞれの範囲
     slice_axes : list(int)
         データ軸がxyzのどの方向に対応しているか表すリスト(0: t, 1: z, 2: y, 3: x)
-    axisunit : UnitTranslator or None
+    axisunits : list(UnitTranslator) or None
         軸の単位変換器
     valunit : UnitTranslator or None
         値の単位変換器
@@ -521,6 +532,112 @@ class Data(np.ndarray):
         """
         return self.slices[0]
 
+    def axis(self, ax):
+        index = self.slice_axes[ax]
+        axis_slice = self.slices[index]
+        return np.array(*utils.slice2tuple(axis_slice))
+
+    @property
+    def x(self):
+        """x軸.
+
+        Returns
+        -------
+        np.ndarray
+            x軸
+        """
+        return np.arange(*utils.slice2tuple(self.xslice))
+
+    @property
+    def y(self):
+        """y軸.
+
+        Returns
+        -------
+        np.ndarray
+            y軸
+        """
+        return np.arange(*utils.slice2tuple(self.yslice))
+
+    @property
+    def z(self):
+        """z軸.
+
+        Returns
+        -------
+        np.ndarray
+            z軸
+        """
+        return np.arange(*utils.slice2tuple(self.zslice))
+
+    @property
+    def t(self):
+        """t軸.
+
+        Returns
+        -------
+        np.ndarray
+            t軸
+        """
+        slc = self.tslice
+        maxlen = (slc.stop - slc.start) // slc.step
+        return np.array(utils.range_with_slice(self.tslice, maxlen=maxlen))
+
+    @property
+    def x_si(self):
+        """SI単位系でのx軸.
+
+        Returns
+        -------
+        np.ndarray
+            SI単位系でのx軸
+        """
+        return self.axisunits[3].reverse(self.x)
+
+    @property
+    def y_si(self):
+        """SI単位系でのy軸.
+
+        Returns
+        -------
+        np.ndarray
+            SI単位系でのy軸
+        """
+        return self.axisunits[2].reverse(self.y)
+
+    @property
+    def z_si(self):
+        """SI単位系でのz軸.
+
+        Returns
+        -------
+        np.ndarray
+            SI単位系でのz軸
+        """
+        return self.axisunits[1].reverse(self.z)
+
+    @property
+    def t_si(self):
+        """SI単位系でのt軸.
+
+        Returns
+        -------
+        np.ndarray
+            SI単位系でのt軸
+        """
+        return self.axisunits[0].reverse(self.t)
+    
+    @property
+    def val_si(self):
+        """SI単位系での値.
+
+        Returns
+        -------
+        np.ndarray
+            SI単位系での値
+        """
+        return self.valunit.reverse(self)
+
     @ property
     def use_axes(self):
         """データ軸がxyzのどの方向に対応しているか表すリストを返す.
@@ -558,6 +675,98 @@ class Data(np.ndarray):
         """
         raise NotImplementedError()
 
+    def gifplot(self,
+                fig=None,
+                axis=0,
+                show=False,
+                savefilename=None,
+                interval=200,
+                repeat=True,
+                title=None,
+                notitle=False,
+                use_si=False,
+                **kwargs):
+        """gifアニメーションを作成する
+
+        Parameters
+        ----------
+        fig : Figure
+            アニメーションを描画するFigure(Noneの場合新しく作成する), by default None
+        axis : int, optional
+            アニメーションする軸, by default 0
+        show : bool, optional
+            プロットを表示する場合True(ファイルに保存する場合は非表示), by default False
+        savefilename : str, optional
+            保存するファイル名(Noneの場合保存しない), by default None
+        interval : int, optional
+            フレーム間のインターバル(ミリ秒), by default 400
+        repeat : bool
+            アニメーションをループするならTrue, by default True
+        title : str, optional
+            タイトル(Noneの場合データ名(phisp等)), by default None
+        notitle : bool, optional
+            タイトルを付けない場合True, by default False
+        use_si : bool
+            SI単位系を用いる場合True(そうでない場合EMSES単位系を用いる), by default False
+        """
+        def _update(i, vmin, vmax):
+            plt.clf()
+
+            # 指定した軸でスライス
+            slices = [slice(None)] * len(self.shape)
+            slices[axis] = i
+            val = self[tuple(slices)]
+
+            # タイトルの設定
+            if notitle:
+                _title = title if len(title) > 0 else None
+            else:
+                if use_si:  # SI単位系を用いる場合
+                    title_format = title + '({} {})'
+                    ax = self.slice_axes[axis]
+                    axisunit = self.axisunits[ax]
+                    _title = title_format.format(
+                        axisunit.reverse(i), axisunit.unit)
+
+                else:  # EMSES単位系を用いる場合
+                    title_format = title + '({})'
+                    slc = self.slices[self.axisunits[axis]]
+                    maxlen = self.shape[axis]
+                    ax = list(utils.range_with_slice(slc, maxlen=maxlen))
+                    index = ax[i]
+                    _title = title_format.format(index)
+
+            val.plot(vmin=vmin, vmax=vmax, title=_title,
+                     use_si=use_si, **kwargs)
+
+        if title is None:
+            title = self.name
+
+        if use_si:
+            vmin = self.valunit.reverse(self.min())
+            vmax = self.valunit.reverse(self.max())
+        else:
+            vmin = self.min()
+            vmax = self.max()
+
+        if fig is None:
+            fig = plt.figure()
+
+        ani = animation.FuncAnimation(
+            fig,
+            _update,
+            fargs=(vmin, vmax),
+            interval=interval,
+            frames=self.shape[axis],
+            repeat=repeat)
+
+        if savefilename is not None:
+            ani.save(savefilename, writer='quantized-pillow')
+        elif show:
+            plt.show()
+        else:
+            return fig, ani
+
 
 class Data4d(Data):
     """4次元データを管理する.
@@ -676,6 +885,11 @@ class Data2d(Data):
         dpi : int, optional
             解像度(figsizeが指定された場合は無視される), by default 10
 
+        Returns
+        -------
+        AxesImage or None
+            プロットしたimageデータ(保存またはshowした場合None)
+
         Raises
         ------
         Exception
@@ -727,17 +941,19 @@ class Data2d(Data):
         kwargs['title'] = kwargs.get('title', None) or _title
 
         mesh = np.meshgrid(x, y)
-        emplt.plot_2dmap(z, mesh=mesh, **kwargs)
+        img = emplt.plot_2dmap(z, mesh=mesh, **kwargs)
 
         if show:
             plt.show()
+            return None
+        else:
+            return img
 
 
 class Data1d(Data):
     """3次元データの1次元直線を管理する.
     """
     def __new__(cls, input_array, **kwargs):
-
         obj = np.asarray(input_array).view(cls)
 
         if 'xslice' not in kwargs:
@@ -779,6 +995,11 @@ class Data1d(Data):
         title : str, optional
             タイトル, by default None
 
+        Returns
+        -------
+        Line2D or None
+            プロットデータを表す線オブジェクト(保存または show した場合None)
+
         Raises
         ------
         Exception
@@ -808,7 +1029,10 @@ class Data1d(Data):
         kwargs['xlabel'] = kwargs.get('xlabel', None) or _xlabel
         kwargs['ylabel'] = kwargs.get('ylabel', None) or _ylabel
 
-        emplt.plot_line(y, x=x, **kwargs)
+        line = emplt.plot_line(y, x=x, **kwargs)
 
         if show:
             plt.show()
+            return None
+        else:
+            return line
